/* DNA_lsystem_types.h */
#ifndef DNA_LSYSTEM_TYPES_H
#define DNA_LSYSTEM_TYPES_H

#define LSYS_MAX_LEN	256
#define MAX_FILE_LEN	1024	/* should be else where */

#include "DNA_listBase.h"
#include "DNA_meshdata_types.h"	/* for MCol */
#include "DNA_ID.h"

struct Ipo;
struct Object;
struct LVertex;
struct LChar;


typedef struct FlInt {
	int integer;
	float real;
} FlInt;

typedef struct TurtleState {
	struct TurtleState *next, *prev;
	float m[4][4];
	float sx, sy, sz;
	MCol mcol;
	int mati;			/* material index */
	float angle, len;
	/* level of detail */
	float acc_F;			/* accumulator for F commands (Level of Detail) */
	float acc_f;
	/* for geometry creation */
	/* union {
		int integer;
		float real;
	} v;
	*/
	FlInt v;
	char pad[4];
} TurtleState;

/* LRules are converted to SaveRules before saving */
typedef struct SaveRule {
	struct SaveRule *next, *prev;
	char *repl;
	char c[2];
	short flag;
	short lifetime;
	short len;	/* length of repl + '\0' char */
} SaveRule;

typedef struct Lsystem {
	ID id;

	struct BoundBox *bb;

	char filename[1024];	/* needs to stay syncronized with MAX_FILE_LEN=1024 */

	char axiom[256]; /* needs to stay syncronized with LSYS_MAX_LEN=256 */
	struct LChar *compaxiom;
	int complen;
	ListBase rules;
	int numrules;	/* for testing */

	int iter;
	float fperiter;		/* frames per generation */
	float defangle, deflen, defthick;
	float angle_step, len_step, thick_step;
	float defcolor[4];	/* r, g, b, a */
	/* Growing */
	float gentime;
	float growfrac;	
	float growsta, growend;
	short growtype;	/* interpolation type, e.g. linear */
	/* Level of detail */
	char pad2[2];
	float lod_minsize;	/* minimal length of a draw command (F) in screen space */

	struct LChar *prod;		/* production string */
	struct LChar *prodn;		/* new production string after another iteration */
	int plen;	/* length of production string */

	short drawflag;
	short flag;

	TurtleState curts;
	ListBase tstates;
	/* Generators */
	ListBase generators;
	/* Needed for loading */
	ListBase srules;
} Lsystem;

/* ls->growtype */
#define LSYS_LINEAR	0
#define LSYS_RESTRICTED	1
#define LSYS_LOGISTIC	2

/* ls->drawflag */
#define LSYS_DRAWSKEL	1
#define LSYS_DRAWGEOM	2	/* draw attached geometry */
#define LSYS_DRAWIMPL	4	/* draw implants */

/* ls->flag */
#define LSYS_COMPILED	1	/* the axiom and the rules are compiled */

#define LSYS_GROWING	2	/* Grow lsystem in time */
#define LSYS_GROWLEN	4	/* Animate length in growing time */
#define LSYS_GROWANG	8	/* Animate angle in growing time */
#define LSYS_GROWSIZE	16	/* Animate size in time */
#define LSYS_GROWCOLOR	32	/* Animate color of lvertices */

#define LSYS_LEVODET	64	/* Level of Detail */

/* error_flag */
#define LSYS_UNRES_RULE		1
#define LSYS_MISSING_CURLY	2

/* Lgenerator_Mesh.flag */
#define	LSYS_POP		1
#define LSYS_POLY		2

/* flag in LsystemModifierData */
#define LSYS_DISTRIBUTION_MAP	1
#define LSYS_ORIENT_TO_NORMAL	2

#endif
